"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deploy = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const os = require("os");
const path = require("path");
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const kit_1 = require("@salesforce/kit");
const ts_types_1 = require("@salesforce/ts-types");
const chalk = require("chalk");
const sourceCommand_1 = require("../../../sourceCommand");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-source', 'deploy');
class deploy extends sourceCommand_1.SourceCommand {
    constructor() {
        super(...arguments);
        this.lifecycleEventNames = ['predeploy', 'postdeploy'];
    }
    async run() {
        if (this.flags.validatedeployrequestid) {
            // TODO: return this.doDeployRecentValidation();
        }
        const hookEmitter = core_1.Lifecycle.getInstance();
        const cs = await this.createComponentSet({
            sourcepath: ts_types_1.asArray(this.flags.sourcepath),
            manifest: ts_types_1.asString(this.flags.manifest),
            metadata: ts_types_1.asArray(this.flags.metadata),
        });
        await hookEmitter.emit('predeploy', { packageXmlPath: cs.getPackageXml() });
        const results = await cs
            .deploy({
            usernameOrConnection: this.org.getUsername(),
        })
            .start();
        await hookEmitter.emit('postdeploy', results);
        // skip a lot of steps that would do nothing
        if (!this.flags.json) {
            this.print(results);
        }
        return results;
    }
    printComponentFailures(result) {
        if (result.response.status === 'Failed' && result.components) {
            // sort by filename then fullname
            const failures = result.getFileResponses().sort((i, j) => {
                if (i.filePath === j.filePath) {
                    // if the have the same directoryName then sort by fullName
                    return i.fullName < j.fullName ? 1 : -1;
                }
                return i.filePath < j.filePath ? 1 : -1;
            });
            this.ux.log('');
            this.ux.styledHeader(chalk.red(`Component Failures [${failures.length}]`));
            this.ux.table(failures, {
                columns: [
                    { key: 'componentType', label: 'Type' },
                    { key: 'fileName', label: 'File' },
                    { key: 'fullName', label: 'Name' },
                    { key: 'problem', label: 'Problem' },
                ],
            });
            this.ux.log('');
        }
    }
    printComponentSuccess(result) {
        var _a;
        if (result.response.success && ((_a = result.components) === null || _a === void 0 ? void 0 : _a.size)) {
            //  sort by type then filename then fullname
            const files = result.getFileResponses().sort((i, j) => {
                if (i.fullName === j.fullName) {
                    // same metadata type, according to above comment sort on filename
                    if (i.filePath === j.filePath) {
                        // same filename's according to comment sort by fullName
                        return i.fullName < j.fullName ? 1 : -1;
                    }
                    return i.filePath < j.filePath ? 1 : -1;
                }
                return i.type < j.type ? 1 : -1;
            });
            // get relative path for table output
            files.forEach((file) => {
                if (file.filePath) {
                    file.filePath = path.relative(process.cwd(), file.filePath);
                }
            });
            this.ux.log('');
            this.ux.styledHeader(chalk.blue('Deployed Source'));
            this.ux.table(files, {
                columns: [
                    { key: 'fullName', label: 'FULL NAME' },
                    { key: 'type', label: 'TYPE' },
                    { key: 'filePath', label: 'PROJECT PATH' },
                ],
            });
        }
    }
    print(result) {
        this.printComponentSuccess(result);
        this.printComponentFailures(result);
        // TODO: this.printTestResults(result); <- this has WI @W-8903671@
        if (result.response.success && this.flags.checkonly) {
            this.log(messages.getMessage('checkOnlySuccess'));
        }
        return result;
    }
}
exports.deploy = deploy;
deploy.description = messages.getMessage('description');
deploy.examples = messages.getMessage('examples').split(os.EOL);
deploy.requiresProject = true;
deploy.requiresUsername = true;
deploy.flagsConfig = {
    checkonly: command_1.flags.boolean({
        char: 'c',
        description: messages.getMessage('flags.checkonly'),
    }),
    wait: command_1.flags.minutes({
        char: 'w',
        default: kit_1.Duration.minutes(sourceCommand_1.SourceCommand.DEFAULT_SRC_WAIT_MINUTES),
        min: kit_1.Duration.minutes(sourceCommand_1.SourceCommand.MINIMUM_SRC_WAIT_MINUTES),
        description: messages.getMessage('flags.wait'),
    }),
    testlevel: command_1.flags.enum({
        char: 'l',
        description: messages.getMessage('flags.testLevel'),
        options: ['NoTestRun', 'RunSpecifiedTests', 'RunLocalTests', 'RunAllTestsInOrg'],
        default: 'NoTestRun',
    }),
    runtests: command_1.flags.array({
        char: 'r',
        description: messages.getMessage('flags.runTests'),
        default: [],
    }),
    ignoreerrors: command_1.flags.boolean({
        char: 'o',
        description: messages.getMessage('flags.ignoreErrors'),
    }),
    ignorewarnings: command_1.flags.boolean({
        char: 'g',
        description: messages.getMessage('flags.ignoreWarnings'),
    }),
    validateddeployrequestid: command_1.flags.id({
        char: 'q',
        description: messages.getMessage('flags.validateDeployRequestId'),
        exclusive: [
            'manifest',
            'metadata',
            'sourcepath',
            'checkonly',
            'testlevel',
            'runtests',
            'ignoreerrors',
            'ignorewarnings',
        ],
    }),
    verbose: command_1.flags.builtin({
        description: messages.getMessage('flags.verbose'),
    }),
    metadata: command_1.flags.array({
        char: 'm',
        description: messages.getMessage('flags.metadata'),
        exclusive: ['manifest', 'sourcepath'],
    }),
    sourcepath: command_1.flags.array({
        char: 'p',
        description: messages.getMessage('flags.sourcePath'),
        exclusive: ['manifest', 'metadata'],
    }),
    manifest: command_1.flags.filepath({
        char: 'x',
        description: messages.getMessage('flags.manifest'),
        exclusive: ['metadata', 'sourcepath'],
    }),
};
//# sourceMappingURL=deploy.js.map