"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.retrieve = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const os = require("os");
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const kit_1 = require("@salesforce/kit");
const ts_types_1 = require("@salesforce/ts-types");
const chalk_1 = require("chalk");
const sourceCommand_1 = require("../../../sourceCommand");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-source', 'retrieve');
class retrieve extends sourceCommand_1.SourceCommand {
    constructor() {
        super(...arguments);
        this.lifecycleEventNames = ['preretrieve', 'postretrieve'];
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async run() {
        var _a;
        const hookEmitter = core_1.Lifecycle.getInstance();
        const proj = await core_1.SfdxProjectJson.create({});
        const packages = await proj.getPackageDirectories();
        const defaultPackage = packages.find((pkg) => pkg.default);
        const cs = await this.createComponentSet({
            // safe to cast from the flags as an array of strings
            packagenames: ts_types_1.asArray(this.flags.packagenames),
            sourcepath: ts_types_1.asArray(this.flags.sourcepath),
            manifest: ts_types_1.asString(this.flags.manifest),
            metadata: ts_types_1.asArray(this.flags.metadata),
        });
        // emit pre retrieve event
        // needs to be a path to the temp dir package.xml
        await hookEmitter.emit('preretrieve', { packageXmlPath: cs.getPackageXml() });
        const mdapiResult = await cs
            .retrieve({
            usernameOrConnection: this.org.getUsername(),
            merge: true,
            output: (_a = this.flags.sourcepath) !== null && _a !== void 0 ? _a : defaultPackage.path,
            packageNames: ts_types_1.asArray(this.flags.packagenames),
        })
            .start();
        const results = mdapiResult.response;
        await hookEmitter.emit('postretrieve', results);
        if (results.status === 'InProgress') {
            throw new core_1.SfdxError(messages.getMessage('retrieveTimeout', [this.flags.wait.minutes]));
        }
        if (this.flags.json) {
            this.ux.logJson(mdapiResult.getFileResponses());
        }
        else {
            this.printTable(results, true);
        }
        return results;
    }
    /**
     * to print the results table of successes, failures, partial failures
     *
     * @param results what the .deploy or .retrieve method returns
     * @param withoutState a boolean to add state, default to true
     */
    printTable(results, withoutState) {
        const stateCol = withoutState ? [] : [{ key: 'state', label: 'STATE' }];
        this.ux.styledHeader(chalk_1.blue(messages.getMessage('retrievedSourceHeader')));
        if (results.success) {
            const columns = [
                { key: 'fullName', label: 'FULL NAME' },
                { key: 'type', label: 'TYPE' },
                { key: 'fileName', label: 'PROJECT PATH' },
            ];
            this.ux.table(results.fileProperties, { columns: [...stateCol, ...columns] });
        }
        else {
            this.ux.log(messages.getMessage('NoResultsFound'));
        }
        // if (results.status === 'SucceededPartial' && results.successes.length && results.failures.length) {
        //   this.ux.log('');
        //   this.ux.styledHeader(yellow(messages.getMessage('metadataNotFoundWarning')));
        //   results.failures.forEach((warning) => this.ux.log(warning.message));
        // }
    }
}
exports.retrieve = retrieve;
retrieve.description = messages.getMessage('description');
retrieve.examples = messages.getMessage('examples').split(os.EOL);
retrieve.requiresProject = true;
retrieve.requiresUsername = true;
retrieve.flagsConfig = {
    sourcepath: command_1.flags.array({
        char: 'p',
        description: messages.getMessage('flags.sourcePath'),
        exclusive: ['manifest', 'metadata'],
    }),
    wait: command_1.flags.minutes({
        char: 'w',
        default: kit_1.Duration.minutes(sourceCommand_1.SourceCommand.DEFAULT_SRC_WAIT_MINUTES),
        min: sourceCommand_1.SourceCommand.MINIMUM_SRC_WAIT_MINUTES,
        description: messages.getMessage('flags.wait'),
    }),
    manifest: command_1.flags.filepath({
        char: 'x',
        description: messages.getMessage('flags.manifest'),
        exclusive: ['metadata', 'sourcepath'],
    }),
    metadata: command_1.flags.array({
        char: 'm',
        description: messages.getMessage('flags.metadata'),
        exclusive: ['manifest', 'sourcepath'],
    }),
    packagenames: command_1.flags.array({
        char: 'n',
        description: messages.getMessage('flags.packagename'),
    }),
};
//# sourceMappingURL=retrieve.js.map