"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceCommand = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const path = require("path");
const command_1 = require("@salesforce/command");
const source_deploy_retrieve_1 = require("@salesforce/source-deploy-retrieve");
const core_1 = require("@salesforce/core");
class SourceCommand extends command_1.SfdxCommand {
    /**
     * will create one ComponentSet to be deployed/retrieved
     * will combine from all options passed in
     *
     * @param options: FlagOptions where to create ComponentSets from
     */
    async createComponentSet(options) {
        const setAggregator = [];
        // go through options to create a list of ComponentSets
        // we'll then combine all of those to deploy/retrieve
        if (options.sourcepath) {
            options.sourcepath.forEach((filepath) => {
                if (core_1.fs.fileExistsSync(filepath)) {
                    setAggregator.push(...source_deploy_retrieve_1.ComponentSet.fromSource(path.resolve(filepath)));
                }
                else {
                    throw new core_1.SfdxError(`The sourcepath "${filepath}" is not a valid source file path.`);
                }
            });
        }
        if (options.packagenames) {
            // return ComponentSet and use packageNames in the library via `.retrieve` options
            setAggregator.push(...new source_deploy_retrieve_1.ComponentSet([]));
        }
        if (options.manifest) {
            setAggregator.push(...(await source_deploy_retrieve_1.ComponentSet.fromManifestFile(options.manifest, {
                // to create a link to the actual source component we need to have it resolve through all packages
                // to find the matching source metadata
                // this allows us to deploy after
                resolve: process.cwd(),
            })));
        }
        if (options.metadata) {
            options.metadata.forEach((entry) => {
                const splitEntry = entry.split(':');
                const metadata = {
                    type: splitEntry[0],
                    // either -m ApexClass or -m ApexClass:MyApexClass
                    fullName: splitEntry.length === 1 ? '*' : splitEntry[1],
                };
                const cs = new source_deploy_retrieve_1.ComponentSet([metadata]);
                // we need to search the entire project for the matching metadata component
                // no better way than to have it search than process.cwd()
                cs.resolveSourceComponents(process.cwd(), { filter: cs });
                setAggregator.push(...cs);
            });
        }
        return new source_deploy_retrieve_1.ComponentSet(setAggregator);
    }
}
exports.SourceCommand = SourceCommand;
SourceCommand.MINIMUM_SRC_WAIT_MINUTES = 1;
SourceCommand.DEFAULT_SRC_WAIT_MINUTES = 33;
//# sourceMappingURL=sourceCommand.js.map